/*
	Copyright (C) 2004 Christopher E. Miller
	
	This software is provided 'as-is', without any express or implied
	warranty.  In no event will the authors be held liable for any damages
	arising from the use of this software.
	
	Permission is granted to anyone to use this software for any purpose,
	including commercial applications, and to alter it and redistribute it
	freely, subject to the following restrictions:
	
	1. The origin of this software must not be misrepresented; you must not
	   claim that you wrote the original software. If you use this software
	   in a product, an acknowledgment in the product documentation would be
	   appreciated but is not required.
	2. Altered source versions must be plainly marked as such, and must not be
	   misrepresented as being the original software.
	3. This notice may not be removed or altered from any source distribution.
*/


import dwindows;


class InputBoxException: WindowException
{
	this(char[] msg)
	{
		super(msg);
	}
}


class InputBox: Window
{
	alias void delegate(int ok, char[] result) OnComplete;
	const char[] CLASSNAME = "InputBox";
	
	static HINSTANCE hinstance;
	
	Static message;
	Edit result;
	OnComplete _complete = null;
	HBRUSH hbrBg;
	COLORREF textColor;
	
	
	private void setColors()
	{
		hbrBg = GetSysColorBrush(COLOR_BTNFACE);
		textColor = GetSysColor(COLOR_BTNTEXT);
	}
	
	
	void create(char[] caption, int x, int y, int width, int height, HWND hwParent)
	{
		try
		{
			super.create(WS_EX_DLGMODALFRAME | WS_EX_CONTROLPARENT, CLASSNAME, caption, WS_CAPTION | WS_POPUP, x, y, width, height, hwParent, null, hinstance);
		}
		catch(WindowException)
		{
			throw new InputBoxException("Unable to create input box.");
		}
		
		setColors();
		HFONT hfont = cast(HBRUSH)GetStockObject(DEFAULT_GUI_FONT);
		RECT r;
		clientRect(&r);
		
		with(message = new Static)
		{
			create(0, "Input", WS_CHILD | WS_VISIBLE, 5, 5, r.right - 10, 40, this.hwnd(), 0);
			font(hfont, false);
		}
		
		with(result = new Edit)
		{
			create(WS_EX_CLIENTEDGE, "", WS_CHILD | WS_VISIBLE | ES_AUTOHSCROLL | WS_BORDER | WS_TABSTOP, 5, 50, r.right - 10, 23, this.hwnd(), 0);
			font(hfont, false);
		}
		
		HWND hw;
		hw = CreateWindowEx(0, "BUTTON", "OK", WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_DEFPUSHBUTTON, r.right - 75 - 4 - 75 - 4, 80, 75, 23, hwnd(), cast(HMENU)IDOK, null, null);
		SendMessage(hw, WM_SETFONT, (WPARAM)hfont, 0);
		hw = CreateWindowEx(0, "BUTTON", "Cancel", WS_CHILD | WS_VISIBLE | WS_TABSTOP, r.right - 75 - 4, 80, 75, 23, hwnd(), cast(HMENU)IDCANCEL, null, null);
		SendMessage(hw, WM_SETFONT, (WPARAM)hfont, 0);
	}
	
	
	override LRESULT windowProc(UINT msg, WPARAM wparam, LPARAM lparam)
	{
		switch(msg)
		{
			case WM_CTLCOLORSTATIC:
				SetTextColor(cast(HDC)wparam, textColor);
				SetBkMode(cast(HDC)wparam, TRANSPARENT);
				return cast(LPARAM)hbrBg;
			
			case WM_COMMAND:
				switch(cast(short)wparam)
				{
					case IDOK:
						if(_complete)
							_complete(true, result.text());
						destroy();
						break;
					
					case IDCANCEL:
						if(_complete)
							_complete(false, null);
						destroy();
						break;
					
					default: ;
				}
				break;
			
			case WM_SYSCOLORCHANGE:
				setColors();
				break;
			
			case WM_CLOSE:
				if(_complete)
					_complete(false, null);
				destroy();
				break;
			
			default:
				return super.windowProc(msg, wparam, lparam);
		}
		return 0;
	}
	
	
	void setOnComplete(OnComplete dg)
	{
		_complete = dg;
	}
}


static this()
{
	WNDCLASSEX wce;
	wce.style = CS_OWNDC | CS_SAVEBITS;
	wce.cbClsExtra = wce.cbWndExtra = 0;
	wce.hInstance = InputBox.hinstance = GetModuleHandle(null);
	wce.hIcon = wce.hIconSm = null;
	wce.hCursor = LoadCursor(null, IDC_ARROW);
	wce.hbrBackground = cast(HBRUSH)(COLOR_BTNFACE + 1);
	wce.lpszMenuName = null;
	wce.lpszClassName = InputBox.CLASSNAME;
	Window.registerClass(&wce);
}


//result is what's first in the edit box
//ok is set to true if the user clicked the OK button
char[] inputBox(HWND hwParent, char[] _text, char[] title, char[] _result, out int ok)
{
	bit mloop = true;
	int sok;
	
	
	void done(int _ok, char[] _r)
	{
		mloop = false;
		sok = _ok;
		if(_ok)
			_result = _r;
		
		EnableWindow(hwParent, TRUE);
		SetActiveWindow(hwParent);
	}
	
	
	HWND hp = hwParent;
	while((GetWindowLong(hp, GWL_STYLE) & WS_CHILD) && (hp = GetParent(hwParent)) != null)
	{
		hwParent = hp;
	}
	
	HWND hibox;
	with(new InputBox)
	{
		create(title, 0, 0, 300, 114 + GetSystemMetrics(SM_CYCAPTION), hwParent);
		EnableWindow(hwParent, FALSE);
		setOnComplete(&done);
		message.text(_text);
		result.text(_result);
		result.setSel(0, _result.length);
		center();
		show();
		SetFocus(result.hwnd());
		_result = null;
		hibox = hwnd();
	}
	
	MSG msg;
	BOOL mr;
	while(mloop)
	{
		mr = GetMessage(&msg, null, 0, 0);
		if(!mr)
		{
			PostQuitMessage(msg.wParam);
			break;
		}
		else
		{
			if(!IsDialogMessage(hibox, &msg))
			{
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
		}
	}
	
	ok = sok;
	return _result;
}


char[] inputBox(HWND hwParent, char[] _text)
{
	int ok;
	return inputBox(hwParent, _text, "Input", "", ok);
}


char[] inputBox(HWND hwParent, char[] _text, char[] title)
{
	int ok;
	return inputBox(hwParent, _text, title, "", ok);
}


char[] inputBox(HWND hwParent, char[] _text, char[] title, char[] _result)
{
	int ok;
	return inputBox(hwParent, _text, title, _result, ok);
}

