// Modified by Christopher E. Miller
// www.dprogramming.com


/*
 * Copyright (c) 2002
 * Pavel "EvilOne" Minayev
 *
 * Permission to use, copy, modify, distribute and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear
 * in supporting documentation.  Author makes no representations about
 * the suitability of this software for any purpose. It is provided
 * "as is" without express or implied warranty.
 */

module winutil;
 
private import win;

alias void VOID;
alias ubyte BYTE;
alias short INT16;
alias ushort WORD;
alias ushort UINT16;
alias int INT_PTR;
alias uint UINT_PTR;
alias int LONG_PTR;
alias uint ULONG_PTR;
alias uint PROPID;
const BOOL FALSE = 0;
const BOOL TRUE = 1;

alias char* PANSICHAR;
alias wchar* PWIDECHAR;

WORD MAKEWORD(ubyte A, ubyte B)
{
	return cast(WORD)A | cast(WORD)(B << 8);
}

LONG MAKELONG(ushort A, ushort B)
{
	return cast(LONG)A | cast(LONG)(B << 16);
}

WORD HIWORD(int l) { return (WORD)((l >> 16) & 0xFFFF); }
WORD LOWORD(int l) { return (WORD)l; }
int FAILED(int status) { return status < 0; }
int SUCCEEDED(int Status) { return Status >= 0; }

BYTE HIBYTE(SHORT W)
{
	return W >> 8;
}

BYTE LOBYTE(SHORT W)
{
	return W & 0xFF;
}

HANDLE GlobalDiscard(HANDLE h)
{
	return GlobalReAlloc(h, 0, GMEM_MOVEABLE);
}

HANDLE LocalDiscard(HANDLE h)
{
	return LocalReAlloc(h, 0, LMEM_MOVEABLE);
}

BOOL IS_ERROR(HRESULT Status)
{
	return ((int)Status >> 31) == SEVERITY_ERROR;
}

int HRESULT_CODE(HRESULT hr)
{
	return (int)hr & 0xFFFF;
}

int HRESULT_FACILITY(HRESULT hr)
{
	return ((int)hr >> 16) & 0x1FFF;
}

int HRESULT_SEVERITY(HRESULT hr)
{
	return ((int)hr >> 31) & 0x1;
}

HRESULT MAKE_HRESULT(int sev, int fac, int code)
{
	return (HRESULT)((sev << 31) | (fac << 16) | code);
}

HRESULT HRESULT_FROM_WIN32(int x)
{
	return (HRESULT) (x ? (x & 0xFFFF) | (FACILITY_WIN32 << 16) | 0x80000000 : 0);
}

//HRESULT HRESULT_FROM_NT(int x)
//{
//	return x | FACILITY_NT_BIT;
//}

DWORD MAKEROP4(DWORD fore, DWORD back)
{
	return ((back << 8) & 0xFF000000) | fore;
}

ubyte GetKValue(COLORREF cmyk)
{
	return cmyk & 0xFF;
}

ubyte GetYValue(COLORREF cmyk)
{
	return (cmyk >> 8) & 0xFF;
}

ubyte GetMValue(COLORREF cmyk)
{
	return (cmyk >> 16) & 0xFF;
}

ubyte GetCValue(COLORREF cmyk)
{
	return (cmyk >> 24) & 0xFF;
}

COLORREF CMYK(ubyte c, ubyte m, ubyte y, ubyte k)
{
	return k | (y << 8) | (m << 16) | (c << 24);
}

COLORREF RGB(ubyte r, ubyte g, ubyte b)
{
	return r | (g << 8) | (b << 16);
}

COLORREF PALETTERGB(ubyte r, ubyte g, ubyte b)
{
	return 0x02000000 | RGB(r, g, b); 
}

COLORREF PALETTEINDEX(ushort i)
{
	return 0x01000000 | i;
}

ubyte GetRValue(COLORREF rgb)
{
	return rgb & 0xFF;
}

ubyte GetGValue(COLORREF rgb)
{
	return (rgb >> 8) & 0xFF;
}

ubyte GetBValue(COLORREF rgb)
{
	return (rgb >> 16) & 0xFF;
}

WPARAM MAKEWPARAM(ushort l, ushort h)
{
	return MAKELONG(l, h);
}

LPARAM MAKELPARAM(ushort l, ushort h)
{
	return MAKELONG(l, h);
}

LRESULT MAKELRESULT(ushort l, ushort h)
{
	return MAKELONG(l, h);
}

BOOL ExitWindows(DWORD dwReserved, UINT uReserved)
{
	return ExitWindowsEx(EWX_LOGOFF, 0);
}

HWND CreateWindowA(PCHAR b, PCHAR c, DWORD d, int e,
	int f, int g, int h, HWND i, HMENU j, HINST k, POINTER l)
{
	return CreateWindowExA(0, b, c, d, e, f, g, h, i, j, k, l);
}

HWND CreateWindowW(PWIDECHAR b, PWIDECHAR c, DWORD d, int e,
	int f, int g, int h, HWND i, HMENU j, HINST k, POINTER l)
{
	return CreateWindowExW(0, b, c, d, e, f, g, h, i, j, k, l);
}

HWND CreateDialogA(HINST a, PANSICHAR b, HWND c, DLGPROC d)
{
	return CreateDialogParamA(a, b, c, d, 0);	
}

HWND CreateDialogW(HINST a, PWIDECHAR b, HWND c, DLGPROC d)
{
	return CreateDialogParamW(a, b, c, d, 0);	
}

HWND CreateDialogIndirectA(HINST a, DLGTEMPLATE* b, HWND c, DLGPROC d)
{
	return CreateDialogIndirectParamA(a, b, c, d, 0);
}

HWND CreateDialogIndirectW(HINST a, DLGTEMPLATE* b, HWND c, DLGPROC d)
{
	return CreateDialogIndirectParamW(a, b, c, d, 0);
}

int DialogBoxA(HINST a, PANSICHAR b, HWND c, DLGPROC d)
{
	return DialogBoxParamA(a, b, c, d, 0);
}

int DialogBoxW(HINST a, PWIDECHAR b, HWND c, DLGPROC d)
{
	return DialogBoxParamW(a, b, c, d, 0);
}

int DialogBoxIndirectA(HINST a, DLGTEMPLATE* b, HWND c, DLGPROC d)
{
	return DialogBoxIndirectParamA(a, b, c, d, 0);
}

int DialogBoxIndirectW(HINST a, DLGTEMPLATE* b, HWND c, DLGPROC d)
{
	return DialogBoxIndirectParamW(a, b, c, d, 0);
}

extern(C)
{
	void* memset(void*, int, uint);
	void* memmove(void*, void*, uint);
	void* memcpy(void*, void*, uint);
}


void ZeroMemory(void* dest, uint len)
{
	memset(dest, 0, len);
}

void FillMemory(void* dest, uint len, ubyte c)
{
	memset(dest, c, len);
}

void MoveMemory(void* dest, void* src, uint len)
{
	memmove(dest, src, len);
}

void CopyMemory(void* dest, void* src, uint len)
{
	memcpy(dest, src, len);
}


//Tab control "macros"

void TabCtrl_AdjustRect(HWND hwnd, BOOL fLarger, RECT* prc)
{
	SendMessageA(hwnd, TCM_ADJUSTRECT, cast(WPARAM)fLarger, cast(LPARAM)prc);
}


BOOL TabCtrl_DeleteAllItems(HWND hwnd)
{
	return cast(BOOL)SendMessageA(hwnd, TCM_DELETEALLITEMS, 0, 0);
}


BOOL TabCtrl_DeleteItem(HWND hwnd, int iItem)
{
	return cast(BOOL)SendMessageA(hwnd, TCM_DELETEITEM, cast(WPARAM)iItem, 0);
}


int TabCtrl_GetCurFocus(HWND hwnd)
{
	return cast(int)SendMessageA(hwnd, TCM_GETCURFOCUS, 0, 0);
}


int TabCtrl_GetCurSel(HWND hwnd)
{
	return cast(int)SendMessageA(hwnd, TCM_GETCURSEL, 0, 0);
}


HIMAGELIST TabCtrl_GetImageList(HWND hwnd)
{
	return cast(HIMAGELIST)SendMessageA(hwnd, TCM_GETIMAGELIST, 0, 0);
}


BOOL TabCtrl_GetItem(HWND hwnd, int iItem, TC_ITEM* pitem)
{
	return cast(BOOL)SendMessageA(hwnd, TCM_GETITEM, cast(WPARAM)iItem, cast(LPARAM)pitem);
}


int TabCtrl_GetItemCount(HWND hwnd)
{
	return cast(int)SendMessageA(hwnd, TCM_GETITEMCOUNT, 0, 0);
}


BOOL TabCtrl_GetItemRect(HWND hwnd, int iItem, RECT* prc)
{
	return cast(BOOL)SendMessageA(hwnd, TCM_GETITEMRECT, cast(WPARAM)iItem, cast(LPARAM)prc);
}


int TabCtrl_GetRowCount(HWND hwnd)
{
	return cast(int)SendMessageA(hwnd, TCM_GETROWCOUNT, 0, 0);
}


int TabCtrl_GetToolTips(HWND hwnd)
{
	return cast(int)SendMessageA(hwnd, TCM_GETTOOLTIPS, 0, 0);
}


int TabCtrl_HitTest(HWND hwnd, TC_HITTESTINFO* pinfo)
{
	return SendMessageA(hwnd, TCM_HITTEST, 0, cast(LPARAM)pinfo);
}


int TabCtrl_InsertItem(HWND hwnd, int iItem, TC_ITEM* pitem)
{
    return (int)SendMessageA(hwnd, TCM_INSERTITEM, cast(WPARAM)iItem, cast(LPARAM)pitem);
}


void TabCtrl_RemoveImage(HWND hwnd, int iImage)
{
	SendMessageA(hwnd, TCM_REMOVEIMAGE, cast(WPARAM)iImage, 0);
}


void TabCtrl_SetCurFocus(HWND hwnd, int iItem)
{
	SendMessageA(hwnd, TCM_SETCURFOCUS, cast(WPARAM)iItem, 0);
}


int TabCtrl_SetCurSel(HWND hwnd, int iItem)
{
	return cast(int)SendMessageA(hwnd, TCM_SETCURSEL, cast(WPARAM)iItem, 0);
}


BOOL TabCtrl_SetImageList(HWND hwnd, HIMAGELIST himl)
{
	return cast(BOOL)SendMessageA(hwnd, TCM_SETIMAGELIST, 0, cast(LPARAM)himl);
}


BOOL TabCtrl_SetItem(HWND hwnd, int iItem, TC_ITEM* pitem)
{
	return cast(BOOL)SendMessageA(hwnd, TCM_SETITEM, cast(WPARAM)iItem, cast(LPARAM)pitem);
}


BOOL TabCtrl_SetItemExtra(HWND hwnd, int cb)
{
	return cast(BOOL)SendMessageA(hwnd, TCM_SETITEMEXTRA, cast(WPARAM)cb, 0);
}


DWORD TabCtrl_SetItemSize(HWND hwnd, int cx, int cy)
{
	return cast(DWORD)SendMessageA(hwnd, TCM_SETITEMSIZE, 0, MAKELPARAM(cx, cy));
}


void TabCtrl_SetPadding(HWND hwnd, int cx, int cy)
{
	SendMessageA(hwnd, TCM_SETPADDING, 0, MAKELPARAM(cx, cy));
}


void TabCtrl_SetToolTips(HWND hwnd, HWND hwndTT)
{
	SendMessageA(hwnd, TCM_SETTOOLTIPS, cast(WPARAM)hwndTT, 0);
}

